@@ -2,8 +2,8 @@
 
 /*
  * $Author: tom $
- * $Date: 2004/12/23 22:24:08 $
- * $Revision: 1.80 $
+ * $Date: 2009/02/15 23:57:03 $
+ * $Revision: 1.86 $
  */
 
 #define YEAR2INDEX(year) (((year) >= 1900) ? ((year) - 1900) : (year))
@@ -204,10 +204,8 @@ CDKCALENDAR *newCDKCalendar(CDKSCREEN *cdkscreen, int xplace, int yplace, char *
    for (x = 0; x < (int) SIZEOF(bindings); ++x)
       bindCDKObject (vCALENDAR, calendar, bindings[x].from, getcCDKBind, (void *)(long)bindings[x].to);
 
-   /* Register this baby. */
    registerCDKObject (cdkscreen, vCALENDAR, calendar);
 
-   /* Return the calendar pointer. */
    return (calendar);
 }
 
@@ -216,20 +214,18 @@ CDKCALENDAR *newCDKCalendar(CDKSCREEN *cdkscreen, int xplace, int yplace, char *
  */
 time_t activateCDKCalendar (CDKCALENDAR *calendar, chtype *actions)
 {
-   /* Declare local variables. */
    chtype input = 0;
+   boolean functionKey;
    time_t ret	= -1;
 
    /* Draw the widget. */
    drawCDKCalendar (calendar, ObjOf(calendar)->box);
 
-   /* Check if 'actions' is null. */
    if (actions == 0)
    {
       for (;;)
       {
-	 /* Get the input. */
-	 input = getcCDKObject (ObjOf(calendar));
+	 input = getchCDKObject (ObjOf(calendar), &functionKey);
 
 	 /* Inject the character into the widget. */
 	 ret = injectCDKCalendar (calendar, input);
@@ -262,32 +258,32 @@ time_t activateCDKCalendar (CDKCALENDAR *calendar, chtype *actions)
  */
 static int _injectCDKCalendar (CDKOBJS *object, chtype input)
 {
-   CDKCALENDAR *calendar = (CDKCALENDAR *)object;
+   CDKCALENDAR *widget = (CDKCALENDAR *)object;
    /* Declare local variables. */
    int ppReturn = 1;
    int ret = unknownInt;
    bool complete = FALSE;
 
    /* Set the exit type. */
-   setExitType(calendar, 0);
+   setExitType(widget, 0);
 
-   /* Refresh the calendar field. */
-   drawCDKCalendarField (calendar);
+   /* Refresh the widget field. */
+   drawCDKCalendarField (widget);
 
    /* Check if there is a pre-process function to be called. */
-   if (PreProcessFuncOf(calendar) != 0)
+   if (PreProcessFuncOf(widget) != 0)
    {
       /* Call the pre-process function. */
-      ppReturn = PreProcessFuncOf(calendar) (vCALENDAR, calendar, PreProcessDataOf(calendar), input);
+      ppReturn = PreProcessFuncOf(widget) (vCALENDAR, widget, PreProcessDataOf(widget), input);
    }
 
    /* Should we continue? */
    if (ppReturn != 0)
    {
       /* Check a predefined binding. */
-      if (checkCDKObjectBind (vCALENDAR, calendar, input) != 0)
+      if (checkCDKObjectBind (vCALENDAR, widget, input) != 0)
       {
-	 checkEarlyExit(calendar);
+	 checkEarlyExit(widget);
 	 complete = TRUE;
       }
       else
@@ -295,78 +291,84 @@ static int _injectCDKCalendar (CDKOBJS *object, chtype input)
 	 switch (input)
 	 {
 	    case KEY_UP :
-		 decrementCalendarDay (calendar, 7);
+		 decrementCalendarDay (widget, 7);
 		 break;
 
 	    case KEY_DOWN :
-		 incrementCalendarDay (calendar, 7);
+		 incrementCalendarDay (widget, 7);
 		 break;
 
 	    case KEY_LEFT :
-		 decrementCalendarDay (calendar, 1);
+		 decrementCalendarDay (widget, 1);
 		 break;
 
 	    case KEY_RIGHT :
-		 incrementCalendarDay (calendar, 1);
+		 incrementCalendarDay (widget, 1);
 		 break;
 
 	    case KEY_NPAGE :
-		 incrementCalendarMonth (calendar, 1);
+		 incrementCalendarMonth (widget, 1);
 		 break;
 
 	    case 'N' :
-		 incrementCalendarMonth (calendar, 6);
+		 incrementCalendarMonth (widget, 6);
 		 break;
 
 	    case KEY_PPAGE :
-		 decrementCalendarMonth (calendar, 1);
+		 decrementCalendarMonth (widget, 1);
 		 break;
 
 	    case 'P' :
-		 decrementCalendarMonth (calendar, 6);
+		 decrementCalendarMonth (widget, 6);
 		 break;
 
 	    case '-' :
-		 decrementCalendarYear (calendar, 1);
+		 decrementCalendarYear (widget, 1);
 		 break;
 
 	    case '+' :
-		 incrementCalendarYear (calendar, 1);
+		 incrementCalendarYear (widget, 1);
 		 break;
 
 	    case KEY_HOME:
-		 setCDKCalendarDate (calendar, -1, -1, -1);
+		 setCDKCalendarDate (widget, -1, -1, -1);
 		 break;
 
 	    case KEY_ESC :
-		 setExitType(calendar, input);
+		 setExitType(widget, input);
+		 complete = TRUE;
+		 break;
+
+	    case KEY_ERROR :
+		 setExitType(widget, input);
 		 complete = TRUE;
 		 break;
 
 	    case KEY_TAB : case KEY_ENTER :
-		 setExitType(calendar, input);
-		 ret = getCurrentTime (calendar);
+		 setExitType(widget, input);
+		 ret = getCurrentTime (widget);
 		 complete = TRUE;
 		 break;
 
 	    case CDK_REFRESH :
-		 eraseCDKScreen (ScreenOf(calendar));
-		 refreshCDKScreen (ScreenOf(calendar));
+		 eraseCDKScreen (ScreenOf(widget));
+		 refreshCDKScreen (ScreenOf(widget));
 		 break;
 	 }
       }
 
       /* Should we do a post-process? */
-      if (!complete && (PostProcessFuncOf(calendar) != 0))
+      if (!complete && (PostProcessFuncOf(widget) != 0))
       {
-	 PostProcessFuncOf(calendar) (vCALENDAR, calendar, PostProcessDataOf(calendar), input);
+	 PostProcessFuncOf(widget) (vCALENDAR, widget, PostProcessDataOf(widget), input);
       }
    }
 
    if (!complete) {
-      setExitType(calendar, 0);
+      setExitType(widget, 0);
    }
-   ResultOf(calendar).valueInt = ret;
+
+   ResultOf(widget).valueInt = ret;
    return (ret != unknownInt);
 }
 
@@ -444,10 +446,8 @@ static void _drawCDKCalendar (CDKOBJS *object, boolean Box)
 	      calendar->xOffset, TitleLinesOf(calendar) + 2,
 	      calendar->DayName, HORIZONTAL, 0, headerLen);
 
-   /* Refresh the main window. */
-   refreshCDKWindow (calendar->win);
+   wrefresh (calendar->win);
 
-   /* Draw in the calendar field. */
    drawCDKCalendarField (calendar);
 }
 
@@ -458,13 +458,13 @@ static void drawCDKCalendarField (CDKCALENDAR *calendar)
 {
    /* Declare local variables. */
    char *monthName	= calendar->MonthName[calendar->month];
-   int monthNameLength	= (int)strlen (monthName);
    int monthLength	= getMonthLength (calendar->year, calendar->month);
    int yearIndex	= YEAR2INDEX(calendar->year);
    int yearLen		= 0;
    int day		= 1;
    int x, y;
-   char temp[10];
+   int save_y = -1, save_x = -1;
+   char temp[30];
 
    for (x = 1; x <= 6; x++)
    {
@@ -472,6 +472,9 @@ static void drawCDKCalendarField (CDKCALENDAR *calendar)
       {
 	 if (day <= monthLength)
 	 {
+	    int ypos = y * 3;
+	    int xpos = x;
+
 	    chtype marker = calendar->dayAttrib;
 
 	    sprintf(temp, "%02d", day);
@@ -479,24 +482,28 @@ static void drawCDKCalendarField (CDKCALENDAR *calendar)
 	    if (calendar->day == day)
 	    {
 	       marker = calendar->highlight;
+	       save_y = xpos + getbegy(calendar->fieldWin) - getbegy(InputWindowOf(calendar));
+	       save_x = 1;
 	    }
 	    else
 	    {
 	       marker |= getCDKCalendarMarker(calendar, day, calendar->month, yearIndex);
 	    }
-	    writeCharAttrib (calendar->fieldWin, y*3, x,
+	    writeCharAttrib (calendar->fieldWin, ypos, xpos,
 			     temp, marker, HORIZONTAL, 0, 2);
 	 }
 	 day++;
       }
    }
-   refreshCDKWindow (calendar->fieldWin);
+   wrefresh (calendar->fieldWin);
 
    /* Draw the month in. */
    if (calendar->labelWin != 0)
    {
+      sprintf (temp, "%s %d,", monthName, calendar->day);
       writeChar (calendar->labelWin, 0, 0,
-		 monthName, HORIZONTAL, 0, monthNameLength);
+		 temp, HORIZONTAL, 0, strlen(temp));
+      wclrtoeol (calendar->labelWin);
 
       /* Draw the year in. */
       sprintf (temp, "%d", calendar->year);
@@ -505,7 +512,11 @@ static void drawCDKCalendarField (CDKCALENDAR *calendar)
 		 calendar->fieldWidth - yearLen, 0,
 		 temp, HORIZONTAL, 0, yearLen);
 
-      refreshCDKWindow (calendar->labelWin);
+      wmove (calendar->labelWin, 0, 0);
+      wrefresh (calendar->labelWin);
+   } else if (save_y >= 0) {
+      wmove (InputWindowOf(calendar), save_y, save_x);
+      wrefresh (InputWindowOf(calendar));
    }
 }
 
@@ -682,6 +693,9 @@ static void _destroyCDKCalendar (CDKOBJS *object)
       deleteCursesWindow (calendar->shadowWin);
       deleteCursesWindow (calendar->win);
 
+      /* Clean the key bindings. */
+      cleanCDKObjectBindings (vCALENDAR, calendar);
+
       /* Unregister the object. */
       unregisterCDKObject (vCALENDAR, calendar);
    }
